<?php

require_once 'emergencybrake.civix.php';
use CRM_Emergencybrake_ExtensionUtil as E;

/**
 * Implements hook_civicrm_config().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_config/
 */
function emergencybrake_civicrm_config(&$config): void {
  _emergencybrake_civix_civicrm_config($config);
}

/**
 * Implements hook_civicrm_install().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_install
 */
function emergencybrake_civicrm_install(): void {
  _emergencybrake_civix_civicrm_install();
}

/**
 * Implements hook_civicrm_enable().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_enable
 */
function emergencybrake_civicrm_enable(): void {
  _emergencybrake_civix_civicrm_enable();
}

/**
 * Implements hook_civicrm_navigationMenu().
 */
function emergencybrake_civicrm_navigationMenu(&$menu) {
  _emergencybrake_civix_insert_navigation_menu($menu, 'Administer/System Settings', [
    'label' => E::ts('Emergency Brake'),
    'name' => 'emergencybrake_settings',
    'url' => 'civicrm/admin/setting/emergencybrake',
    'permission' => 'administer CiviCRM system',
    'operator' => 'OR',
    'separator' => 0,
  ]);
  _emergencybrake_civix_navigationMenu($menu);
}

function emergencybrake_civicrm_post($op, $objectName, $objectId, &$objectRef): void {
  if ($op == 'create' && $objectName == 'Contribution') {
    $maxfails = \Civi::settings()->get('emergencybrake_maxfails') ?? 5;
    // Note that credit cards get created with Pending and may not fail until
    // later, so let's check back 5 contributions not including this one.
    $contributions = \Civi\Api4\Contribution::get(FALSE)
      ->addSelect('contribution_status_id:name')
      ->addWhere('id', '<>', $objectId)
      ->addOrderBy('id', 'DESC')
      ->setLimit($maxfails)
      ->execute();
    $fails = 0;
    foreach ($contributions as $contribution) {
      if ($contribution['contribution_status_id:name'] != 'Failed') {
        break;
      }
      $fails++;
    }
    if ($fails == $maxfails) {
      CRM_Core_DAO::executeQuery("UPDATE civicrm_payment_processor SET is_active = 0");
      $emails = \Civi::settings()->get('emergencybrake_emails') ?? '';
      if (!empty($emails)) {
        $firstEmail = explode(',', $emails)[0];
        $domainEmail = \CRM_Core_BAO_Domain::getNameAndEmail();
        $mailParams = array(
          'from' => $domainEmail[1],
          'toName' => $firstEmail,
          'toEmail' => $firstEmail,
          'cc' => $emails,
          'subject' => E::ts('ACTION REQUIRED: Multiple consecutive failed transactions'),
          'text' => E::ts('No text version'),
          'html' => E::ts('Please visit %1 and note that all payment processors have been disabled. You will need to re-enable manually.', [1 => CRM_Utils_System::url('civicrm/contribute', '', TRUE)]),
        );
        CRM_Utils_Mail::send($mailParams);
      }
    }
  }
}
